package com.netmera.netmera_flutter_sdk;

import android.app.Activity;
import android.content.ContentValues;
import android.content.Context;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.netmera.Netmera;
import com.netmera.NetmeraCategory;
import com.netmera.NetmeraError;
import com.netmera.NetmeraInbox;
import com.netmera.NetmeraInboxCategory;
import com.netmera.NetmeraInteractiveAction;
import com.netmera.NetmeraPushObject;
import com.netmera.callbacks.NMCategoryPreferenceFetchCallback;
import com.netmera.callbacks.NMCategoryPreferenceSetCallback;
import com.netmera.callbacks.NMInboxCountResultListener;
import com.netmera.data.NMCategoryPreference;
import com.netmera.data.NMInboxStatusCount;
import com.netmera.data.NMInboxStatusCountFilter;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;

/**
 * Copyright (c) 2019 Inomera Research.
 *
 * @author Burak Soykal
 */
public class FNetmera {

    private static final String USER_ID = "userId";
    private static final String EMAIL = "email";
    private static final String MSISDN = "msisdn";
    private static final String CODE = "code";

    private static final String ERROR_CODE_NULL_INBOX = "2017";
    private static final String ERROR_MESSAGE_NULL_INBOX = "Must call fetchInbox method first";

    private static final String ERROR_CODE_NOT_NEXT_PAGE = "2018";
    private static final String ERROR_MESSAGE_NOT_NEXT_PAGE = "Not next page";

    private static final String ERROR_CODE_UPDATE_STATUS = "2019";
    private static final String ERROR_MESSAGE_UPDATE_STATUS = "There was a problem updating status";

    private static final String ERROR_CODE_NULL_CATEGORY = "2020";
    private static final String ERROR_MESSAGE_NULL_CATEGORY = "Must call fetchCategory method first";

    private static final String ERROR_CODE_INVALID_PUSH_OBJECT = "2021";
    private static final String ERROR_MESSAGE_INVALID_PUSH_OBJECT = "Received NetmeraPushObject was not valid.";

    private static final String ERROR_CODE_NULL_INBOX_STATUS = "2022";
    private static final String ERROR_MESSAGE_NULL_INBOX_STATUS = "Inbox status can't be null.";

    private static final String ERROR_CODE_INBOX_STATUS = "2023";
    private static final String ERROR_MESSAGE_INBOX_STATUS = "Error occurred while fetching inbox count for status.";

    private static final String ERROR_CODE_CATEGORY_PREFERENCE = "2024";
    private static final String ERROR_MESSAGE_CATEGORY_PREFERENCE = "Error occurred while fetching user category preference list.";

    private static final String ERROR_CODE_SET_CATEGORY_PREFERENCE = "2025";
    private static final String ERROR_MESSAGE_SET_CATEGORY_PREFERENCE = "Error occurred while setting user category preference list.";

    private static NetmeraInbox netmeraInbox;
    private static NetmeraInboxCategory netmeraInboxCategory;

    public static void initNetmera(FNetmeraConfiguration fNetmeraConfiguration) {
        Netmera.init(fNetmeraConfiguration.getNetmeraConfiguration());
        setNetmeraHeaders();
    }

    static void setBaseUrl(MethodCall call) {
        String baseUrl = call.argument("baseUrl");
        Netmera.setBaseUrl(baseUrl);
    }

    static void requestPermissionsForLocation() {
        Netmera.requestPermissionsForLocation();
    }

    static void setNetmeraMaxActiveRegions(MethodCall call) {
        int maxActiveRegions = call.argument("maxActiveRegions");
        Netmera.setNetmeraMaxActiveRegions(maxActiveRegions);
    }

    static void currentExternalId(MethodChannel.Result result) {
        result.success(Netmera.getCurrentExternalId());
    }

    static void enablePopupPresentation() {
        Netmera.enablePopupPresentation();
    }

    static void disablePopupPresentation() {
        Netmera.disablePopupPresentation();
    }

    static void enablePush() {
        Netmera.enablePush();
    }

    static void disablePush() {
        Netmera.disablePush();
    }

    static void isPushEnabled(MethodChannel.Result result) {
        result.success(Netmera.isPushEnabled());
    }

    static void turnOffSendingEventAndUserUpdate(MethodCall call) {
        boolean turnOff = call.argument("turnOff");
        Netmera.turnOffSendingEventAndUserUpdate(turnOff);
    }

    static void fetchInbox(final MethodCall call, final MethodChannel.Result result) {
        Netmera.fetchInbox(FNetmeraInboxFilter.getInboxObject(call), new NetmeraInbox.NetmeraInboxFetchCallback() {
            @Override
            public void onFetchInbox(NetmeraInbox netmeraInbox, NetmeraError netmeraError) {
                setInbox(netmeraInbox, netmeraError, result);
            }
        });
    }

    static void fetchNextPage(final MethodChannel.Result result) {
        if (netmeraInbox == null) {
            result.error(ERROR_CODE_NULL_INBOX, ERROR_MESSAGE_NULL_INBOX, null);
            return;
        }

        if (netmeraInbox.hasNextPage()) {
            netmeraInbox.fetchNextPage(new NetmeraInbox.NetmeraInboxFetchCallback() {
                @Override
                public void onFetchInbox(NetmeraInbox netmeraInbox, NetmeraError error) {
                    setInbox(netmeraInbox, error, result);
                }
            });
        } else {
            result.error(ERROR_CODE_NOT_NEXT_PAGE, ERROR_MESSAGE_NOT_NEXT_PAGE, null);
        }
    }

    static void setInbox(NetmeraInbox inbox, NetmeraError netmeraError, MethodChannel.Result result) {
        if (netmeraError != null) {
            result.error(ERROR_CODE_NULL_INBOX, ERROR_MESSAGE_NULL_INBOX, null);
            return;
        }
        netmeraInbox = inbox;
        result.success(FNetmeraPushObject.mapPushObjects(inbox.pushObjects()));
    }

    static void countForStatus(MethodCall call, MethodChannel.Result result) {
        int status = call.argument("status");
        if (netmeraInbox == null) {
            result.error(ERROR_CODE_NULL_INBOX, ERROR_MESSAGE_NULL_INBOX, null);
            return;
        }
        result.success(netmeraInbox.countForStatus(status));
    }

    static void inboxUpdateStatus(final MethodCall call, final MethodChannel.Result result) {
        ArrayList<String> inboxList = call.argument("inboxList");
        int status = call.argument("status");
        if (netmeraInbox == null || inboxList == null) {
            result.error(ERROR_CODE_NULL_INBOX, ERROR_MESSAGE_NULL_INBOX, null);
            return;
        }

        ArrayList<NetmeraPushObject> selectedInboxList = new ArrayList<>();
        for (String pushId : inboxList) {
            for (NetmeraPushObject inbox : netmeraInbox.pushObjects()) {
                if (inbox.getPushId().equals(pushId)) {
                    selectedInboxList.add(inbox);
                }
            }
        }

        netmeraInbox.updateStatus(selectedInboxList, status, new NetmeraInbox.NetmeraInboxStatusCallback() {
            @Override
            public void onSetStatusInbox(NetmeraError netmeraError) {
                if (netmeraError != null) {
                    result.error(ERROR_CODE_UPDATE_STATUS, ERROR_MESSAGE_UPDATE_STATUS, netmeraError.getMessage());
                } else {
                    result.success(true);
                }
            }
        });
    }

    static void sendEvent(MethodCall call) {
        Map<String, Object> eventMap = call.arguments();
        eventMap.values().removeAll(Collections.singleton(null));
        FNetmeraEvent event = new FNetmeraEvent();

        if (hasKey(eventMap, CODE)) {
            event.setCode((String) eventMap.get(CODE));
            eventMap.remove(CODE);
        }

        event.setEventParameters(eventMap);
        Netmera.sendEvent(event);
    }

    static void updateUser(MethodCall call) {
        Map<String, Object> userMap = call.arguments();
        userMap.values().removeAll(Collections.singleton(null));
        FNetmeraUser netmeraUser = new FNetmeraUser();

        if (hasKey(userMap, USER_ID)) {
            netmeraUser.setUserId((String) userMap.get(USER_ID));
            userMap.remove(USER_ID);
        }

        if (hasKey(userMap, EMAIL)) {
            netmeraUser.setEmail((String) userMap.get(EMAIL));
            userMap.remove(EMAIL);
        }

        if (hasKey(userMap, MSISDN)) {
            netmeraUser.setMsisdn((String) userMap.get(MSISDN));
            userMap.remove(MSISDN);
        }

        netmeraUser.setUserParameters(userMap);
        Netmera.updateUser(netmeraUser);
    }

    static void setApiKey(final MethodCall call) {
        String apiKey = call.argument("apiKey");
        Netmera.setApiKey(apiKey);
    }

    static void handlePushObject(Activity activity, MethodCall call, MethodChannel.Result result) {
        String pushId = call.argument("pushId");

        if (netmeraInbox == null) {
            result.error(ERROR_CODE_NULL_INBOX, ERROR_MESSAGE_NULL_INBOX, null);
            return;
        }

        for (NetmeraPushObject pushObject : netmeraInbox.pushObjects()) {
            if (pushObject.getPushId().equals(pushId)) {
                Netmera.handlePushObject(activity, pushObject);
                break;
            }
        }
    }

    static void handleLastMessage(Activity activity, MethodCall call, MethodChannel.Result result) {
        String categoryName = call.argument("categoryName");

        if (netmeraInboxCategory == null) {
            result.error(ERROR_CODE_NULL_CATEGORY, ERROR_MESSAGE_NULL_CATEGORY, null);
            return;
        }

        if (categoryName == null || categoryName.isEmpty()) {
            result.error(ERROR_CODE_INVALID_PUSH_OBJECT, ERROR_MESSAGE_INVALID_PUSH_OBJECT, null);
            return;
        }

        for (NetmeraCategory category : netmeraInboxCategory.categories()) {
            if (categoryName.equals(category.getCategoryName())) {
                Netmera.handlePushObject(activity, category.getLastMessage());
                break;
            }
        }
    }

    private static void setNetmeraHeaders() {
        ContentValues headerValues = new ContentValues();
        headerValues.put("X-netmera-framework", "flutter");
        headerValues.put("X-netmera-frameworkV", "2.1.10");
        Netmera.setNetmeraHeaders(headerValues);
    }

    static void initializeService(Context context, Object args, MethodChannel.Result result) {
        HashMap arguments = (HashMap) args;

        saveToSharedPref(context, NetmeraFlutterSdkPlugin.CALLBACK_DISPATCHER_HANDLE_KEY, arguments.get(NetmeraFlutterSdkPlugin.CALLBACK_DISPATCHER_HANDLE_KEY));
        saveToSharedPref(context, FNetmeraPushBroadcastReceiver.ON_PUSH_REGISTER, arguments.get(FNetmeraPushBroadcastReceiver.ON_PUSH_REGISTER));
        saveToSharedPref(context, FNetmeraPushBroadcastReceiver.ON_PUSH_RECEIVE, arguments.get(FNetmeraPushBroadcastReceiver.ON_PUSH_RECEIVE));
        saveToSharedPref(context, FNetmeraPushBroadcastReceiver.ON_PUSH_DISMISS, arguments.get(FNetmeraPushBroadcastReceiver.ON_PUSH_DISMISS));
        saveToSharedPref(context, FNetmeraPushBroadcastReceiver.ON_PUSH_OPEN, arguments.get(FNetmeraPushBroadcastReceiver.ON_PUSH_OPEN));
        saveToSharedPref(context, FNetmeraPushBroadcastReceiver.ON_PUSH_BUTTON_CLICKED, arguments.get(FNetmeraPushBroadcastReceiver.ON_PUSH_BUTTON_CLICKED));

        // Start worker and check if there is a message waiting to be sent to the flutter side.
        FNetmeraWorker.createAndStart(context, null);

        result.success(true);
    }

    private static boolean hasKey(Map map, String key) {
        return map.containsKey(key) && map.get(key) != null;
    }

    private static void saveToSharedPref(Context context, String key, Object value) {
        context.getSharedPreferences(NetmeraFlutterSdkPlugin.SHARED_PREFERENCES_KEY, Context.MODE_PRIVATE)
                .edit()
                .putLong(key, Long.parseLong(value.toString()))
                .apply();
    }

    static void fetchCategory(final MethodCall call, final MethodChannel.Result result) {
        Netmera.fetchCategory(FNetmeraCategoryFilter.getInboxObject(call), new NetmeraInboxCategory.NetmeraInboxCategoryCallback() {
            @Override
            public void onFetchCategory(NetmeraInboxCategory netmeraInboxCategory, NetmeraError netmeraError) {
                setCategory(netmeraInboxCategory, netmeraError, result);
            }
        });
    }

    static void fetchNextCategoryPage(final MethodChannel.Result result) {
        if (netmeraInboxCategory == null) {
            result.error(ERROR_CODE_NULL_CATEGORY, ERROR_MESSAGE_NULL_CATEGORY, null);
            return;
        }

        if (netmeraInboxCategory.hasNextPage()) {
            netmeraInboxCategory.fetchNextPage(new NetmeraInboxCategory.NetmeraInboxCategoryCallback() {
                @Override
                public void onFetchCategory(NetmeraInboxCategory netmeraInboxCategory, NetmeraError netmeraError) {
                    setCategory(netmeraInboxCategory, netmeraError, result);
                }
            });
        } else {
            result.error(ERROR_CODE_NOT_NEXT_PAGE, ERROR_MESSAGE_NOT_NEXT_PAGE, null);
        }
    }

    private static void setCategory(NetmeraInboxCategory nmInboxCategory, NetmeraError netmeraError, MethodChannel.Result result) {
        if (netmeraError != null) {
            result.error(ERROR_CODE_NULL_INBOX, ERROR_MESSAGE_NULL_INBOX, netmeraError.getMessage());
            return;
        }
        netmeraInboxCategory = nmInboxCategory;
        result.success(FNetmeraCategoryObject.mapCategoryObjects(netmeraInboxCategory.categories()));
    }

    static void handleInteractiveAction(Context context, final MethodCall call, MethodChannel.Result result) {
        String interactiveActionId = call.argument("interactiveActionId");

        if (netmeraInbox == null) {
            result.error(ERROR_CODE_NULL_INBOX, ERROR_MESSAGE_NULL_INBOX, null);
            return;
        }

        for (NetmeraPushObject pushObject : netmeraInbox.pushObjects()) {
            if (pushObject.getInteractiveActions() != null) {
                for (NetmeraInteractiveAction action : pushObject.getInteractiveActions()) {
                    if (action.getId().equals(interactiveActionId)) {
                        Netmera.handleInteractiveAction(context, action);
                        return;
                    }
                }
            }
        }
    }

    static void updateStatusByCategories(final MethodCall call, final MethodChannel.Result result) {
        ArrayList<String> categoryList = call.argument("categoryList");
        int inboxStatus = call.argument("inboxStatus");

        Netmera.updateStatusByCategories(inboxStatus, categoryList, new NetmeraInbox.NetmeraInboxStatusCallback() {
            @Override
            public void onSetStatusInbox(NetmeraError netmeraError) {
                if (netmeraError != null) {
                    result.error(ERROR_CODE_UPDATE_STATUS, ERROR_MESSAGE_UPDATE_STATUS, netmeraError.getMessage());
                } else {
                    result.success(true);
                }
            }
        });
    }

    static void updateAll(final MethodCall call, final MethodChannel.Result result) {
        int inboxStatus = call.argument("inboxStatus");

        Netmera.updateAll(inboxStatus, new NetmeraInbox.NetmeraInboxStatusCallback() {
            @Override
            public void onSetStatusInbox(NetmeraError netmeraError) {
                if (netmeraError != null) {
                    result.error(ERROR_CODE_UPDATE_STATUS, ERROR_MESSAGE_UPDATE_STATUS, null);
                } else {
                    result.success(true);
                }
            }
        });
    }

    static void getInboxCountForStatus(final MethodCall call, final MethodChannel.Result result) {
        NMInboxStatusCountFilter.Builder builder = new NMInboxStatusCountFilter.Builder();
        Map<String, Object> filterMap = call.arguments();

        if (!hasKey(filterMap, "status")) {
            result.error(ERROR_CODE_NULL_INBOX_STATUS, ERROR_MESSAGE_NULL_INBOX_STATUS, null);
            return;
        } else {
            builder.setInboxStatus(FNMInboxStatusCount.getInboxStatus((int) filterMap.get("status")));
        }

        if (hasKey(filterMap, "categoryList")) {
            builder.setCategoryList((Integer[]) filterMap.get("categoryList"));
        }

        if (hasKey(filterMap, "includeExpired")) {
            builder.setIncludeExpired((boolean) filterMap.get("includeExpired"));
        }

        Netmera.getInboxCountForStatus(builder.build(), new NMInboxCountResultListener() {
            @Override
            public void onSuccess(@NonNull NMInboxStatusCount nmInboxStatusCount) {
                result.success(FNMInboxStatusCount.parseInboxCountForStatusToMap(nmInboxStatusCount));
            }

            @Override
            public void onFailure(@Nullable String s) {
                result.error(ERROR_CODE_INBOX_STATUS, ERROR_MESSAGE_INBOX_STATUS, null);
            }
        });
    }

    static void getUserCategoryPreferenceList(final MethodCall call, final MethodChannel.Result result) {
        Netmera.getUserCategoryPreferenceList(new NMCategoryPreferenceFetchCallback() {
            @Override
            public void onSuccess(List<NMCategoryPreference> list) {
                result.success(FNMUserCategoryPreference.parseUserCategoryPreferenceListToMap(list));
            }

            @Override
            public void onFailure(@Nullable String s) {
                result.error(ERROR_CODE_CATEGORY_PREFERENCE, ERROR_MESSAGE_CATEGORY_PREFERENCE, null);
            }
        });
    }

    static void setUserCategoryPreference(final MethodCall call, final MethodChannel.Result result) {
        int categoryId = call.argument("categoryId");
        boolean categoryEnabled = call.argument("categoryEnabled");

        Netmera.setUserCategoryPreference(categoryId, categoryEnabled, new NMCategoryPreferenceSetCallback() {
            @Override
            public void onSuccess() {
                result.success(true);
            }

            @Override
            public void onFailure(@Nullable String s) {
                result.error(ERROR_CODE_SET_CATEGORY_PREFERENCE, ERROR_MESSAGE_SET_CATEGORY_PREFERENCE, null);
            }
        });
    }

    static void onNetmeraPushMessageReceived(final MethodCall call) {
        Map<String, String> map = call.arguments();
        Bundle bundle = new Bundle();

        if (map.containsKey("google.c.sender.id")) {
            for (String key : map.keySet()) {
                bundle.putString(key, map.get(key));
            }
            Netmera.onNetmeraPushMessageReceived(new com.google.firebase.messaging.RemoteMessage(bundle));
        } else {
            com.huawei.hms.push.RemoteMessage.Builder builder = new com.huawei.hms.push.RemoteMessage.Builder("to");
            builder.setData(map);
            Netmera.onNetmeraPushMessageReceived(builder.build());
        }
    }

    static void onNetmeraNewToken(MethodCall call) {
        String pushToken = call.argument("pushToken");
        Netmera.onNetmeraNewToken(pushToken);
    }

}
