package com.netmera.netmera_flutter_sdk;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.os.Bundle;
import android.util.Log;

import androidx.annotation.NonNull;

import com.google.firebase.messaging.RemoteMessage;
import com.netmera.Netmera;

import java.nio.BufferUnderflowException;
import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

/**
 * NetmeraFlutterSdkPlugin
 */
public class NetmeraFlutterSdkPlugin implements MethodCallHandler, FlutterPlugin, ActivityAware {

    @SuppressLint("StaticFieldLeak")
    private static Activity activity;
    static final String SHARED_PREFERENCES_KEY = "netmera_plugin_cache";
    static final String CALLBACK_DISPATCHER_HANDLE_KEY = "callback_dispatch_handler";

    private static final String CURRENT_EXTERNAL_ID = "currentExternalId";
    private static final String SET_BASE_URL = "setBaseUrl";
    private static final String REQUEST_PERMISSIONS_FOR_LOCATION = "requestPermissionsForLocation";
    private static final String SET_NETMERA_MAX_ACTIVE_REGIONS = "setNetmeraMaxActiveRegions";
    private static final String ENABLE_POPUP_PRESENTATION = "enablePopupPresentation";
    private static final String DISABLE_POPUP_PRESENTATION = "disablePopupPresentation";
    private static final String ENABLE_PUSH = "enablePush";
    private static final String DISABLE_PUSH = "disablePush";
    private static final String IS_PUSH_ENABLED = "isPushEnabled";
    private static final String TURN_OFF_SENDING_EVENT_AND_USER_UPDATE = "turnOffSendingEventAndUserUpdate";
    private static final String FETCH_INBOX = "fetchInbox";
    private static final String FETCH_NEXT_PAGE = "fetchNextPage";
    private static final String COUNT_FOR_STATUS = "countForStatus";
    private static final String INBOX_UPDATE_STATUS = "inboxUpdateStatus";
    private static final String FETCH_CATEGORY = "fetchCategory";
    private static final String FETCH_NEXT_CATEGORY = "fetchNextCategory";
    private static final String SEND_EVENT = "sendEvent";
    private static final String UPDATE_USER = "updateUser";
    private static final String INIT_SERVICE = "initService";
    private static final String SET_API_KEY = "setApiKey";
    private static final String HANDLE_PUSH_OBJECT = "handlePushObject";
    private static final String HANDLE_LAST_MESSAGE = "handleLastMessage";
    private static final String HANDLE_INTERACTIVE_ACTION = "handleInteractiveAction";
    private static final String UPDATE_STATUS_BY_CATEGORIES = "updateStatusByCategories";
    private static final String UPDATE_ALL = "updateAll";
    private static final String GET_INBOX_COUNT_FOR_STATUS = "getInboxCountForStatus";
    private static final String GET_USER_CATEGORY_PREFERENCE_LIST = "getUserCategoryPreferenceList";
    private static final String SET_USER_CATEGORY_PREFERENCE = "setUserCategoryPreference";

    private static final String ON_NETMERA_PUSH_MESSAGE_RECEIVED = "onNetmeraPushMessageReceived";
    private static final String ON_NETMERA_NEW_TOKEN = "onNetmeraNewToken";

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
        final MethodChannel channel = new MethodChannel(binding.getBinaryMessenger(), "netmera_flutter_sdk");
        channel.setMethodCallHandler(new NetmeraFlutterSdkPlugin());
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        activity = binding.getActivity();
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {
    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
    }

    @Override
    public void onDetachedFromActivity() {
        activity = null;
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        switch (call.method) {
            case CURRENT_EXTERNAL_ID:
                FNetmera.currentExternalId(result);
                break;
            case SET_BASE_URL:
                FNetmera.setBaseUrl(call);
                break;
            case REQUEST_PERMISSIONS_FOR_LOCATION:
                FNetmera.requestPermissionsForLocation();
                break;
            case SET_NETMERA_MAX_ACTIVE_REGIONS:
                FNetmera.setNetmeraMaxActiveRegions(call);
                break;
            case ENABLE_POPUP_PRESENTATION:
                FNetmera.enablePopupPresentation();
                break;
            case DISABLE_POPUP_PRESENTATION:
                FNetmera.disablePopupPresentation();
                break;
            case ENABLE_PUSH:
                FNetmera.enablePush();
                break;
            case DISABLE_PUSH:
                FNetmera.disablePush();
                break;
            case IS_PUSH_ENABLED:
                FNetmera.isPushEnabled(result);
                break;
            case TURN_OFF_SENDING_EVENT_AND_USER_UPDATE:
                FNetmera.turnOffSendingEventAndUserUpdate(call);
                break;
            case FETCH_INBOX:
                FNetmera.fetchInbox(call, result);
                break;
            case FETCH_NEXT_PAGE:
                FNetmera.fetchNextPage(result);
                break;
            case COUNT_FOR_STATUS:
                FNetmera.countForStatus(call, result);
                break;
            case INBOX_UPDATE_STATUS:
                FNetmera.inboxUpdateStatus(call, result);
                break;
            case FETCH_CATEGORY:
                FNetmera.fetchCategory(call, result);
                break;
            case FETCH_NEXT_CATEGORY:
                FNetmera.fetchNextCategoryPage(result);
                break;
            case SEND_EVENT:
                FNetmera.sendEvent(call);
                break;
            case UPDATE_USER:
                FNetmera.updateUser(call);
                break;
            case INIT_SERVICE:
                FNetmera.initializeService(activity.getApplicationContext(), call.arguments, result);
                break;
            case SET_API_KEY:
                FNetmera.setApiKey(call);
                break;
            case HANDLE_PUSH_OBJECT:
                FNetmera.handlePushObject(activity, call, result);
                break;
            case HANDLE_LAST_MESSAGE:
                FNetmera.handleLastMessage(activity, call, result);
                break;
            case HANDLE_INTERACTIVE_ACTION:
                FNetmera.handleInteractiveAction(activity.getApplicationContext(), call, result);
                break;
            case UPDATE_STATUS_BY_CATEGORIES:
                FNetmera.updateStatusByCategories(call, result);
                break;
            case UPDATE_ALL:
                FNetmera.updateAll(call, result);
                break;
            case GET_INBOX_COUNT_FOR_STATUS:
                FNetmera.getInboxCountForStatus(call, result);
                break;
            case GET_USER_CATEGORY_PREFERENCE_LIST:
                FNetmera.getUserCategoryPreferenceList(call, result);
                break;
            case SET_USER_CATEGORY_PREFERENCE:
                FNetmera.setUserCategoryPreference(call, result);
                break;
            case ON_NETMERA_PUSH_MESSAGE_RECEIVED:
                FNetmera.onNetmeraPushMessageReceived(call);
                break;
            case ON_NETMERA_NEW_TOKEN:
                FNetmera.onNetmeraNewToken(call);
                break;
        }
    }
}
